// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package usecase

import (
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/usecase/repo"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/log"
	"golang.org/x/net/context"
)

type eventUsecase struct {
	userUC            User
	eventRepo         repo.EventRepository
	expenseRepo       repo.ExpenseRepository
	participationRepo repo.ParticipationRepository
	transactionRepo   repo.TransactionRepository

	dbRepo repo.DBRepository
}

// For the controller
type Event interface{}

func NewEventUsecase(
	uuc User,
	ev repo.EventRepository,
	ex repo.ExpenseRepository,
	pa repo.ParticipationRepository,
	tr repo.TransactionRepository,
	db repo.DBRepository,
) Event {
	return &eventUsecase{uuc, ev, ex, pa, tr, db}
}

func (evuc *eventUsecase) CreateEvent(
	ctx context.Context,
	evDTO *model.EventCreateDTO,
) (*model.EventInfoVO, error) {
	// transfer evDTO to PO

	evPO := &model.EventPO{
		Name:            evDTO.Name,
		Description:     evDTO.Description,
		OwnerID:         evDTO.OwnerID,
		TotalAmount:     0,
		DefaultCurrency: string(evDTO.DefaultCurrency),
	}

	data, err := evuc.dbRepo.Transaction(
		ctx,
		func(txCtx context.Context, tx interface{}) (interface{}, error) {
			created, err := evuc.eventRepo.Create(ctx, evPO)
			if err != nil {
				return nil, err
			}

			// TODO: App log, maybe can be sent to some third party service.
			log.InfoLog(
				"created new event",
				"name",
				created.Name,
				"owner",
				created.OwnerID,
			)

			ownerVO, err := evuc.userUC.GetUserBaseVOByID(ctx, created.OwnerID)
			if err != nil {
				return nil, err
			}

			evVO := &model.EventInfoVO{
				ID:          created.ID,
				Name:        created.Name,
				Description: created.Description,
				TotalAmount: model.MakeMoney(
					created.TotalAmount,
					model.Currency(created.DefaultCurrency),
				),
				Owner:     ownerVO,
				CreatedAt: created.CreatedAt,
			}
			return evVO, err
		})
	if err != nil {
		return nil, err
	}

	res := data.(*model.EventInfoVO)

	return res, err
}
